@@ -51,6 +51,19 @@
                         (vma)->vm_end - (vma)->vm_start, \
                         (vma)->vm_page_prot)
 
+/* Page size*/
+#ifndef PAGE_SIZE_4K
+#define PAGE_SIZE_4K (4UL*1024)
+#endif
+#ifndef PAGE_SIZE_2M
+#define PAGE_SIZE_2M (2UL*1024*1024)
+#endif
+#ifndef PAGE_SIZE_4M
+#define PAGE_SIZE_4M (4UL*1024*1024)
+#endif
+#ifndef PAGE_SIZE_1G
+#define PAGE_SIZE_1G (1UL*1024*1024*1024)
+#endif
 
 /* Page table macros */
 
@@ -75,27 +88,29 @@ do { \
 } while(0)
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,11)
-
-#define PMD_OFFSET(pmd_p, pgd_p, pte_linear) \
+#define PUD_PRESENT(pud_p) \
 do { \
-    pud_t *pud_p = pud_offset(pgd_p, pte_linear); \
-    if (!pud_present(*(pud_p)))	\
+    if (!pud_present(*(pud_p))) \
     { \
         return PAGING_FAULT_SIGBUS_INT;   /* Something bad happened; generate SIGBUS */ \
         /* alternatively we could generate a NOPAGE_OOM "out of memory" */ \
     } \
-    pmd_p = pmd_offset(pud_p, pte_linear); \
 } while(0)
 
-#else /* LINUX_VERSION_CODE < KERNEL_VERSION(2,6,11) */
+#define PUD_OFFSET(pud_p, pgd_p, pte_linear)  \
+do { \
+    pud_p = pud_offset(pgd_p, pte_linear); \
+} while(0)
+
+#define PUD_HUGE(pud) ((pud_val(pud) & _PAGE_PSE) != 0)
+
+#endif
 
 #define PMD_OFFSET(pmd_p, pgd_p, pte_linear)	\
 do { \
     pmd_p = pmd_offset(pgd_p, pte_linear); \
 } while(0)
 
-#endif
-
 #define PMD_PRESENT(pmd_p) \
 do { \
     if (!pmd_present(*(pmd_p)))	\
@@ -107,6 +122,7 @@ do { \
 
 #define PMD_HUGE(pmd) ((pmd_val(pmd) & _PAGE_PSE) != 0)
 
+
 #ifdef pte_offset_atomic
 #define PTE_OFFSET(pte, pmd_p, pte_linear) \
 do { \
@@ -181,7 +197,6 @@ struct firegl_pcie_mem;
 
 
 // note: assigning uniqe types to originally non interchangeable types
-typedef struct { int uniqe3; } kcl_sigset_t;
 typedef struct { int uniqe7; } kcl_file_operations_t;
 
 typedef	int (*kcl_read_proc_t)(
@@ -200,11 +215,10 @@ typedef struct {
     int                     privdevcount; // number of privdev structs
     kcl_proc_list_t *      proclist;
     const char *            name;
-	unsigned int	        major_version;
-	unsigned int	        minor_version;
-	unsigned int            patchlevel;
-	const char *	        date;
-    kcl_sigset_t *         psigmask;
+    unsigned int	        major_version;
+    unsigned int	        minor_version;
+    unsigned int            patchlevel;
+    const char *	        date;
     void *                  privglobal; // private global context
 } kcl_device_t;
 
@@ -409,12 +423,6 @@ extern const unsigned int* ATI_API_CALL mc_heap_get_page_idx_list(void *pointer_
 
 /*****************************************************************************/
 
-extern void ATI_API_CALL KCL_SIGNAL_ConfigureMask(kcl_sigset_t *pSigMask);
-extern int ATI_API_CALL firegl_sig_notifier(void *priv);
-extern int              KCL_SIGNAL_Notifier(void *priv);
-extern void ATI_API_CALL KCL_SIGNAL_BlockAll(int (*notifier)(void *priv), void *pPriv, kcl_sigset_t *pSigMask);
-extern void ATI_API_CALL KCL_SIGNAL_UnblockAll(void);
-
 extern unsigned long ATI_API_CALL kcl__cmpxchg(volatile void *ptr, unsigned long old,                      
                       unsigned long new, int size);
 
@@ -475,9 +483,10 @@ extern void ATI_API_CALL KCL_MEM_IncPageCount_Mapping(void* page);
 extern int ATI_API_CALL KCL_MEM_VerifyReadAccess(void* addr, kcl_size_t size);
 extern int ATI_API_CALL KCL_MEM_VerifyWriteAccess(void* addr, kcl_size_t size);
 extern unsigned long ATI_API_CALL KCL_GetPageTableByVirtAddr(unsigned long virtual_addr, unsigned long* page_addr);
+extern unsigned int ATI_API_CALL KCL_GetPageSizeByVirtAddr(unsigned long virtual_addr, unsigned int* page_size);
 extern int ATI_API_CALL KCL_LockUserPages(unsigned long vaddr, unsigned long* page_list, unsigned int page_cnt);
 extern void ATI_API_CALL KCL_UnlockUserPages(unsigned long* page_list, unsigned int page_cnt);
-extern int ATI_API_CALL KCL_TestAndClearPageDirtyFlag(unsigned long virtual_addr);
+extern int ATI_API_CALL KCL_TestAndClearPageDirtyFlag(unsigned long virtual_addr, unsigned int page_size);
 extern unsigned long ATI_API_CALL KCL_MEM_AllocLinearAddrInterval(KCL_IO_FILE_Handle  file, unsigned long addr, unsigned long len, unsigned long pgoff);
 extern int ATI_API_CALL KCL_MEM_ReleaseLinearAddrInterval(unsigned long addr, unsigned long len);
 extern void* ATI_API_CALL KCL_MEM_MapPageList(unsigned long *pagelist, unsigned int count);
@@ -547,8 +556,6 @@ extern int ATI_API_CALL KCL_MEM_VM_MapRegion(KCL_IO_FILE_Handle filp,
 
 extern int ATI_API_CALL firegl_pci_save_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
 extern int ATI_API_CALL firegl_pci_restore_state(KCL_PCI_DevHandle pdev, struct drm_device* dev);
-extern int ATI_API_CALL firegl_cail_powerdown(struct drm_device* dev, int state);
-extern int ATI_API_CALL firegl_cail_powerup(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_save_framebuffer(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_restore_framebuffer(struct drm_device* dev);
 extern void ATI_API_CALL firegl_pm_save_onchip_ram(struct drm_device* dev);
@@ -805,6 +812,8 @@ extern void ATI_API_CALL KCL_spin_lock(void *lock);
 extern void ATI_API_CALL KCL_spin_unlock(void *lock);
 extern void ATI_API_CALL KCL_get_random_bytes(void *buf, int nbytes);
 extern void* ATI_API_CALL KCL_get_pubdev(void);
+extern void  ATI_API_CALL KCL_fpu_begin(void);
+extern void  ATI_API_CALL KCL_fpu_end(void);
 
 //The length of uuid, standardized by the Open Software Foundation, is 16.
 //In kernel, hard coded in function generate_random_uuid(). 
@@ -830,7 +839,13 @@ extern int libip_iommu_invalid_pri_request( KCL_PCI_DevHandle pdev,
                                             unsigned long fault_addr,
                                             KCL_IOMMU_req_perm_t perm);
 
-extern int libip_iommu_vas_term( KCL_PCI_DevHandle pdev, int  pasid);
+extern int libip_iommu_invalidate_pasid_ctx( KCL_PCI_DevHandle pdev, int  pasid);
+
+//libip call back entry when suspend. return 0 if success . 
+extern int ATI_API_CALL libip_suspend(struct drm_device* dev, int state);
+
+//libip call back entry when resume. return 0 if success . 
+extern int ATI_API_CALL libip_resume(struct drm_device* dev);
 
 
 #endif /* _FIREGL_PUBLIC_H_ */
